#include "stdafx.h"
#include <assert.h>

#ifdef _MSC_VER
// #pragma comment (lib, "Psapi.lib")
#endif

#define MOZILLA_STRICT_API

#include "../../../gecko-sdk/include/nsCOMPtr.h"
#include "../../../gecko-sdk/include/nsIModule.h"
#include "../../../gecko-sdk/include/nsIFactory.h"
#include "../../../gecko-sdk/include/nsIGenericFactory.h"
#include "../../../gecko-sdk/include/nsIComponentManager.h"
#include "../../../gecko-sdk/include/nsIComponentRegistrar.h"
#include "../../../gecko-sdk/include/nsIServiceManager.h"
#include "../../../gecko-sdk/include/nsICategoryManager.h"
#include "../../../gecko-sdk/include/nsMemory.h"
#include "../../../gecko-sdk/include/nsStringAPI.h"

#include "IStatusbarExCore.h"
#include "sys_query.h"



#if 0
class StatusbarExCore : public IStatusbarExCore
{
public:

};
#endif


static const char *s_szClassName = "StatusbarExCore";
static const char *s_szContractID = "@doudehou/statusbarEx;1";

// the CID
// {9C876432-4901-4a41-ACB4-20D3550E5655}
#define STATUSBAREXCORE_CID \
{ 0x9c876432, 0x4901, 0x4a41, \
{ 0xac, 0xb4, 0x20, 0xd3, 0x55, 0xe, 0x56, 0x55 } }





//////////////////////////////////////////////////////////////////////////
class StatusbarExCore : public IStatusbarExCore
{
public:
	NS_DECL_ISUPPORTS
	NS_DECL_ISTATUSBAREXCORE

	StatusbarExCore();

private:
	virtual ~StatusbarExCore();

protected:
	/* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(StatusbarExCore, IStatusbarExCore)

StatusbarExCore::StatusbarExCore()
{
	/* member initializers and constructor code */
}

StatusbarExCore::~StatusbarExCore()
{
	/* destructor code */
}

/* void GetMemoryStatus (out long total_memory, out long free_memory); */
NS_IMETHODIMP StatusbarExCore::GetMemoryStatus(PRInt32 *total_memory, PRInt32 *free_memory)
{
	assert (total_memory);
	assert (free_memory);

	sys_info_query *eq = sys_info_query::get_instance();
	eq->get_global_memory_status(total_memory, free_memory);

	return NS_OK;
}


/* void GetFxMemory (out long memory_usage); */
NS_IMETHODIMP StatusbarExCore::GetFxMemory(PRInt32 *usage, PRInt32 *vm_size)
{
	assert (usage && vm_size);

	sys_info_query *eq = sys_info_query::get_instance();
	eq->get_fx_memory(usage, vm_size);

	return NS_OK;
}


/* void GetFxCpuUsage (out long cpu_usage); */
NS_IMETHODIMP StatusbarExCore::GetFxCpuUsage(PRInt32 *cpu_usage)
{
	assert (cpu_usage);

	sys_info_query *eq = sys_info_query::get_instance();
	*cpu_usage = eq->get_fx_cpu_usage();
	return NS_OK;
}




/* long GetEthernetCount (); */
NS_IMETHODIMP StatusbarExCore::GetEthernetCount(PRInt32 *_retval)
{
	assert (_retval);

	sys_info_query *eq = sys_info_query::get_instance();
	*_retval = eq->get_ethernet_count();
	return NS_OK;
}

/* void GetEthernetSpeed (in long index, out long in_speed, out long out_speed); */
NS_IMETHODIMP StatusbarExCore::GetEthernetSpeed(PRInt32 index, PRInt32 *in_speed, PRInt32 *out_speed)
{
	sys_info_query *eq = sys_info_query::get_instance();
	*in_speed = eq->get_in_speed(index);
	*out_speed = eq->get_out_speed(index);
	return NS_OK;
}

/* void GetEthernetName (in long index, out DOMString name); */
NS_IMETHODIMP StatusbarExCore::GetEthernetName(PRInt32 index, nsAString & name)
{
	sys_info_query *eq = sys_info_query::get_instance();
	std::wstring std_name;
	eq->get_ethernet_name(index, std_name);
	NS_StringSetData(name, std_name.c_str());
	return NS_OK;
}



NS_IMETHODIMP StatusbarExCore::GetPowerStatus(
					PRInt32 *ACLineStatus, 
					PRInt32 *BatteryFlag, 
					PRInt32 *BatteryLifePercent, 
					PRInt32 *BatteryLifeTime,
					PRInt32 *hour, 
					PRInt32 *minute, 
					PRInt32 *second
					)
{
	SYSTEM_POWER_STATUS sps;
	memset(&sps, 0, sizeof(sps));
	if (!GetSystemPowerStatus(&sps))
		return NS_ERROR_FAILURE;
	// GetSystemPowerStatus(&sps);

	if (ACLineStatus)
		*ACLineStatus = sps.ACLineStatus;
	if (BatteryFlag)
		*BatteryFlag = sps.BatteryFlag;
	if (BatteryLifePercent)
		*BatteryLifePercent = sps.BatteryLifePercent;
	if (BatteryLifeTime)
		*BatteryLifeTime = sps.BatteryLifeTime;

	DWORD dw = sps.BatteryLifeTime;
	if (dw == (DWORD) -1)
		dw = 0;
	if (hour)
		*hour = dw / 3600;
	if (minute)
		*minute = (dw % 3600) / 60;
	if (second)
		*second = (dw % 60);

	return NS_OK;
}




//////////////////////////////////////////////////////////////////////////
// the factory

NS_GENERIC_FACTORY_CONSTRUCTOR(StatusbarExCore)
static NS_METHOD StatusbarExCoreRegistration(nsIComponentManager *aCompMgr,
					     nsIFile *aPath,
					     const char *registryLocation,
					     const char *componentType,
					     const nsModuleComponentInfo *info);
static NS_METHOD StatusbarExCoreUnregistration(nsIComponentManager *aCompMgr,
					       nsIFile *aPath,
					       const char *registryLocation,
					       const nsModuleComponentInfo *info);



static const nsModuleComponentInfo s_arComponents[] =
{
	{
		s_szClassName,
		STATUSBAREXCORE_CID,
		s_szContractID,
		StatusbarExCoreConstructor,
		StatusbarExCoreRegistration,
		StatusbarExCoreUnregistration
	},
};


NS_IMPL_NSGETMODULE(StatusbarExCoreModule, s_arComponents)




static NS_METHOD StatusbarExCoreRegistration(nsIComponentManager *aCompMgr,
					nsIFile *aPath,
					const char *registryLocation,
					const char *componentType,
					const nsModuleComponentInfo *info)
{
	nsresult rv;

	nsCOMPtr<nsIServiceManager> servman =
		do_QueryInterface((nsISupports*)aCompMgr, &rv);

	if (NS_FAILED(rv))
		return rv;

	nsCOMPtr<nsICategoryManager> catman;
	servman->GetServiceByContractID(NS_CATEGORYMANAGER_CONTRACTID,
		NS_GET_IID(nsICategoryManager),
		getter_AddRefs(catman));

	if (NS_FAILED(rv))
		return rv;


	char* previous = nsnull;
	rv = catman->AddCategoryEntry("xpcom-startup",
		s_szClassName,
		s_szContractID,
		PR_TRUE,
		PR_TRUE,
		&previous);
	if (previous)
		nsMemory::Free(previous);
	return rv;
}

static NS_METHOD StatusbarExCoreUnregistration(nsIComponentManager *aCompMgr,
					  nsIFile *aPath,
					  const char *registryLocation,
					  const nsModuleComponentInfo *info)
{
	nsresult rv;

	nsCOMPtr<nsIServiceManager> servman =
		do_QueryInterface((nsISupports*)aCompMgr, &rv);
	if (NS_FAILED(rv))
		return rv;

	nsCOMPtr<nsICategoryManager> catman;
	servman->GetServiceByContractID(NS_CATEGORYMANAGER_CONTRACTID,
		NS_GET_IID(nsICategoryManager),
		getter_AddRefs(catman));
	if (NS_FAILED(rv))
		return rv;

	rv = catman->DeleteCategoryEntry("xpcom-startup",
		s_szClassName,
		PR_TRUE);
	return rv;
}
